/**HEADER********************************************************************
* 
* Copyright (c) 2010 Avnet Japan Co.,Ltd (NIKE Borad Project.)
* All Rights Reserved
*
*************************************************************************** 
*
* $FileName: sensor.c$
* $Version : 0.1.0.0$
* $Date    : Jan-29-2010$
*
* Comments:
*
*   Sensor( Acceleration, temp EEEE etc.) driver functions
*
*END************************************************************************/


#include "mqx.h"
#include "fio.h"
#include "fio_prv.h"
#include "io.h"
#include "sensor.h"
#include "sensor_prv.h"
#include "sensor_conf.h"


_mqx_int _sensor_adc_trigger(SENSOR_ADC_CHANNEL_GENERIC_PTR channel, SENSOR_ADC_TRIGGER_MASK mask);
_mqx_int _sensor_adc_pause(SENSOR_ADC_CHANNEL_GENERIC_PTR channel, SENSOR_ADC_TRIGGER_MASK mask);
_mqx_int _sensor_adc_resume(SENSOR_ADC_CHANNEL_GENERIC_PTR channel, SENSOR_ADC_TRIGGER_MASK mask);
_mqx_int _sensor_adc_stop(SENSOR_ADC_CHANNEL_GENERIC_PTR channel, SENSOR_ADC_TRIGGER_MASK mask);

/* configuration of driver, channel(s) independent */
SENSOR_ADC_PTR pSensor_adc = NULL;
/* channel dependent configurations */
SENSOR_ADC_CHANNEL_PTR pSensor_adc_ch[SENSOR_ADC_MAX_CHANNELS] = { 0 };

/* scheduler constants */
static SENSOR_ADC_TRIGGER_MASK Running_mask = 0;



/*FUNCTION*****************************************************************
* 
* Function Name    : _io_sensor_install
* Returned Value   : _mqx_uint a task error code or MQX_OK
* Input Value      : [IN] A string that identifies the device for fopen
* Comments         :
*    Install a sensor driver.
*
*END*********************************************************************/

_mqx_uint _io_sensor_install(
      char_ptr  identifier
)
{ /* Body */
   _mqx_uint result;

   result = _io_dev_install(identifier,
      _sensor_open,
      _sensor_close,
      _sensor_read,
      _sensor_write,
      _sensor_ioctl,
      NULL); 

      return result;

} /* Endbody */


/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_open
* Returned Value   : _mqx_int - IO_ERROR or IO_OK
*     [IN] the file handle for the device being opened
*      FILE_PTR   fd_ptr,
*       
*      [IN] the remaining portion of the name of the device
*      char_ptr   open_name_ptr,
*
*      [IN] the initialization parameters (SENSOR_INIT_STRUCT_PTR)
*      char_ptr   flags
* Comments         : Opens and initializes Sensor (ADC) driver.
* 
*END*********************************************************************/
_mqx_int _sensor_open(
      FILE_PTR   fd_ptr,
      char_ptr   open_name_ptr,
      char_ptr   flags
)
{ /* Body */
   char_ptr file_name_ptr = fd_ptr->DEV_PTR->IDENTIFIER; /* use driver name temporarily */
   _mqx_int status;

   while (*file_name_ptr++ != 0)
      open_name_ptr++; /* move to the file name */
   file_name_ptr = open_name_ptr; /* throw out file_name_ptr and assign the correct file name */

   if (*file_name_ptr == 0) {
      /* 
      ** SENSOR (ADC) DEVICE DRIVER OPENED WITHOUT CHANNEL SPECIFIED
      */
      SENSOR_INIT_STRUCT_PTR init_from = (SENSOR_INIT_STRUCT_PTR) flags;

      if (init_from == NULL){
         return IO_ERROR; /* mandatory open param missing */
      }

      if (NULL == pSensor_adc) {
         pSensor_adc = (SENSOR_ADC_PTR) _mem_alloc_zero(sizeof(SENSOR_ADC));
#if MQX_CHECK_MEMORY_ALLOCATION_ERRORS
         if (pSensor_adc == NULL){
            return IO_ERROR; /* memory alllocation error */
         }
#endif
      } else{
         return IO_ERROR; /* cannot open more than one instance of ADC driver */
      }
      
      pSensor_adc->g.resolution = init_from->resolution;
      pSensor_adc->g.triggers_fire = 0; /* no trigger fired */
      pSensor_adc->g.run = 0;

      if (IO_OK != (status = _sensor_adc_hw_init(pSensor_adc))) { /* init cpu according to adc_general_config */
         _mem_free(pSensor_adc);
         pSensor_adc = NULL;
      }
      
      //fd_ptr->DEV_DATA_PTR = NULL; /* not needed: we dont need to store any additional data */

      /* add 11/19/09  GPIO init */
      _sensor_port_hw_init(pSensor_adc);

      return status;
   }
   else {
      /*
      ** CHANNEL TO BE OPENED, NUMBER IS IN OPEN_NAME_PTR
      */
      SENSOR_ADC_INIT_CHANNEL_STRUCT_PTR init_from = (SENSOR_ADC_INIT_CHANNEL_STRUCT_PTR) flags;
      _mqx_uint                   ch = 0;
      _mqx_uint                   radix = 1;

      if (init_from == NULL){
         return IO_ERROR; /* mandatory open param missing */
      }

      if (NULL == pSensor_adc){
         return IO_ERROR; /* general device was not opened */
      }

      if (init_from->number_samples < 1){
         return IO_ERROR; /* number of samples must be more than 0 */
      }

      /* move through digits to the end of string */
//      open_name_ptr = file_name_ptr; // not needed, already done
      while ((*open_name_ptr >= '0') && (*open_name_ptr <= '9'))
         open_name_ptr++;

      if (*open_name_ptr != 0){
         /* if any non-digit character was found */
         return IO_ERROR;
      }

      open_name_ptr--; /* move to the last digit in string */

      do {
         if (SENSOR_ADC_MAX_CHANNELS <= (ch += radix * (*open_name_ptr - '0'))){
            return IO_ERROR; /* the number of channel is too high */
         }
         radix *= 10;
      } while (open_name_ptr-- != file_name_ptr); /* move in the string to the start of channel number */

      /* channel_number now contains valid channel number */

      if (pSensor_adc_ch[ch] == NULL) {
         pSensor_adc_ch[ch] = (SENSOR_ADC_CHANNEL_PTR) _mem_alloc_zero(sizeof(SENSOR_ADC_CHANNEL));
#if MQX_CHECK_MEMORY_ALLOCATION_ERRORS
         if (pSensor_adc_ch[ch] == NULL){
            return IO_ERROR; /* memory allocation error */
         }
#endif
      } else {
         return IO_ERROR; /* channel already in use */
      }

      /* allocate channel buffer */
      pSensor_adc_ch[ch]->g.buffer_start = _mem_alloc_zero((init_from->buffer_size + 1) * sizeof(SENSOR_ADC_RESULT_STRUCT)); /* zero is needed for ADC_CHANNEL_ACCUMULATE */
      pSensor_adc_ch[ch]->g.buffer_end = pSensor_adc_ch[ch]->g.buffer_start + (init_from->buffer_size + 1);

      if (pSensor_adc_ch[ch]->g.buffer_start == NULL) {
         _mem_free (pSensor_adc_ch[ch]);
         pSensor_adc_ch[ch] = NULL;
         return IO_ERROR; /* memory allocation error */
      }

      // initialize channel
      pSensor_adc_ch[ch]->g.buffer_task = pSensor_adc_ch[ch]->g.buffer_start;
      pSensor_adc_ch[ch]->g.buffer_driver = pSensor_adc_ch[ch]->g.buffer_start;

      pSensor_adc_ch[ch]->g.number = ch;
      pSensor_adc_ch[ch]->g.source = init_from->source;
      pSensor_adc_ch[ch]->g.init_flags = init_from->flags;
      pSensor_adc_ch[ch]->g.trigger = init_from->trigger;
      pSensor_adc_ch[ch]->g.offset = init_from->time_offset;
      pSensor_adc_ch[ch]->g.period = init_from->time_period;
      
      
      pSensor_adc_ch[ch]->g.samples = 0;
      pSensor_adc_ch[ch]->g.samples_preset = init_from->number_samples - 1;
#if MQX_USE_LWEVENTS
      pSensor_adc_ch[ch]->g.complete_event = init_from->complete_event;
#endif

      if (IO_OK != (status = _sensor_adc_hw_channel_init(ch))) { /* this should initialize ADC hardware */
          _mem_free(pSensor_adc_ch[ch]->g.buffer_start);
          _mem_free (pSensor_adc_ch[ch]);
          pSensor_adc_ch[ch] = NULL;
          return status;
      }

      /* store file context for opened channel */
      fd_ptr->DEV_DATA_PTR = pSensor_adc_ch[ch]; /* store information about channel */

      /* if channel should be run in opening file phase, then run it using special trigger */
      if (!(init_from->flags & SENSOR_ADC_CHANNEL_START_TRIGGERED)) {
          return _sensor_ioctl(fd_ptr, IOCTL_SENSOR_ADC_RUN_CHANNEL, NULL);
      }
   }

   return IO_OK;

} /* Endbody */


/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_close
* Returned Value   : ERROR CODE
* Comments         : Closes ADC driver or channel
* 
*END*********************************************************************/

_mqx_int _sensor_close
   (
      /* [IN] the file handle for the device being closed */
      FILE_PTR   fd_ptr
   )
{ /* Body */
   SENSOR_ADC_CHANNEL_GENERIC_PTR channel = (SENSOR_ADC_CHANNEL_GENERIC_PTR) fd_ptr->DEV_DATA_PTR;
   _mqx_int        i, status;

   for (i = 0, status = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
      if (pSensor_adc_ch[i] != NULL) {
         status++;
         if ((pointer) pSensor_adc_ch[i] == (pointer) channel) {
            _sensor_adc_stop(channel, 0);
            if (IO_OK != (status = _sensor_adc_hw_channel_deinit( (SENSOR_ADC_CHANNEL_PTR) channel))){
               return status;
            }

            _mem_free(channel->buffer_start);
            _mem_free(channel);
            pSensor_adc_ch[i] = NULL;
            return IO_OK;
         }
      }
   }
   
   /* if we reach this code, then we close ADC */
   /* test, if there is still any open channel */
   if (status != 0){
         return IO_ERROR; /* cannot close ADC driver: there are still open channels */
   }

   if (IO_OK != (status = _sensor_adc_hw_deinit())){
     return status;
   }

   _mem_free(pSensor_adc);
   pSensor_adc = NULL;

   return IO_OK;

} /* Endbody */


/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_read
* Returned Value   : number of characters read
* Comments         : Reads data from ADC driver
* 
*END*********************************************************************/

_mqx_int _sensor_read
   (
      /* [IN] the file handle for the device */
      FILE_PTR   fd_ptr,

      /* [IN] where the characters are to be stored */
      char_ptr   data_ptr,

      /* [IN] the number of characters to input */
      _mqx_int   num
   )
{ /* Body */
    _mqx_int cp_cnt;
    uint_32_ptr dst = (uint_32_ptr)data_ptr;
    SENSOR_ADC_CHANNEL_GENERIC_PTR ch_conf = (SENSOR_ADC_CHANNEL_GENERIC_PTR) fd_ptr->DEV_DATA_PTR;
    
    if (ch_conf->buffer_task == NULL){
        return 0; /* channel not run yet */
    }

//    printf("\n####Call _sensor_read( %s)", data_ptr);

    cp_cnt = (ch_conf->buffer_driver - ch_conf->buffer_task);
    if (cp_cnt < 0){
        cp_cnt += (ch_conf->buffer_end - ch_conf->buffer_start);
    }
    
    if (cp_cnt > (num / sizeof(uint_32))){
        cp_cnt = (num / sizeof(uint_32));
    }
    
    while (cp_cnt) {
        *dst++ = ch_conf->buffer_task->result;
        ch_conf->buffer_task->result = 0; /* zero buffer to be usable next time (applicable for ADC_CHANNEL_ACCUMULATE) */
        if (++ch_conf->buffer_task >= ch_conf->buffer_end){
            ch_conf->buffer_task = ch_conf->buffer_start;
        }
        cp_cnt--;
    }
    
    return ((char_ptr)dst - data_ptr);
} /* Endbody */


/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_write
* Returned Value   : number of characters written
* Comments         : Does nothing, ADC is solely read-only driver
* 
*END*********************************************************************/

_mqx_int _sensor_write
   (
      /* [IN] the file handle for the device */
      FILE_PTR   fd_ptr,

      /* [IN] where the characters are */
      char_ptr   data_ptr,

      /* [IN] the number of characters to output */
      _mqx_int   num
   )
{ /* Body */
   return 0;
} /* Endbody */


/*FUNCTION*****************************************************************
* 
* Function Name    : _adc_ioctl
* Returned Value   : int_32
* Comments         :
*    Returns result of ioctl operation.
*
*END*********************************************************************/

_mqx_int _sensor_ioctl
   (
      /* [IN] the file handle for the device */
      FILE_PTR   fd_ptr,

      /* [IN] the ioctl command */
      _mqx_uint  cmd,

      /* [IN] the ioctl parameters */
      pointer    param_ptr
   )
{ /* Body */
    SENSOR_ADC_CHANNEL_GENERIC_PTR adc_ch = fd_ptr->DEV_DATA_PTR;
    uint_32_ptr             uparam_ptr = param_ptr;

    switch (cmd) {
        case IOCTL_SENSOR_ADC_RUN_CHANNEL:
            /* Runs specific channel */
            return _sensor_adc_trigger(adc_ch, 0);
        case IOCTL_SENSOR_ADC_FIRE_TRIGGER:
            /* Runs channels associated with trigger mask */
            return _sensor_adc_trigger(NULL, (SENSOR_ADC_TRIGGER_MASK)param_ptr);
        case IOCTL_SENSOR_ADC_STOP_CHANNEL:
            /* Stops specific channel */
            return _sensor_adc_stop(adc_ch, 0);
        case IOCTL_SENSOR_ADC_STOP_CHANNELS:
            /* Stops channels associated with trigger mask */
            return _sensor_adc_stop(NULL, (SENSOR_ADC_TRIGGER_MASK)param_ptr);
        case IOCTL_SENSOR_ADC_PAUSE_CHANNEL:
            /* Stops specific channel */
            return _sensor_adc_pause(adc_ch, 0);
        case IOCTL_SENSOR_ADC_PAUSE_CHANNELS:
            /* Stops channels associated with trigger mask */
            return _sensor_adc_pause(NULL, (SENSOR_ADC_TRIGGER_MASK)param_ptr);
        case IOCTL_SENSOR_ADC_RESUME_CHANNEL:
            /* Resumes specific channel */
            return _sensor_adc_resume(adc_ch, 0);
        case IOCTL_SENSOR_ADC_RESUME_CHANNELS:
            /* Resumes channels associated with trigger mask */
            return _sensor_adc_resume(NULL, (SENSOR_ADC_TRIGGER_MASK)param_ptr);

        case IO_IOCTL_DEVICE_IDENTIFY:  
            /* return the device identify */
            uparam_ptr[0] = IO_DEV_TYPE_PHYS_ADC;
            uparam_ptr[1] = 0;
            uparam_ptr[2] = IO_DEV_ATTR_READ;
            return MQX_OK;          

        /* 11/19/2009 add */
        case IOCTL_SENSOR_PORT_SET:
            return _sensor_port_set( (uchar)param_ptr);

        case IOCTL_SENSOR_PORT_CLR:
            return _sensor_port_clr( (uchar)param_ptr);

        case IOCTL_SENSOR_PORT_IN:
            return _sensor_port_in( (uchar)param_ptr);

        case IOCTL_SENSOR_GET_PORT:
            {
              SENSOR_PORT_INFO_PTR pPortInfo = (SENSOR_PORT_INFO_PTR)param_ptr;
              return _sensor_port_get( pPortInfo->type, &pPortInfo->data);
            }

            //return _sensor_port_get( (uchar)(((SENSOR_PORT_INFO_PTR)param_ptr)->type), (uchar_ptr)(&((SENSOR_PORT_INFO_PTR)param_ptr->data));

        default:
            break;
            //return IO_ERROR_INVALID_IOCTL_CMD;
    }
    return IO_ERROR_INVALID_IOCTL_CMD;
} /* Endbody */

/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_adc_scheduler
* Returned Value   : 
* Comments         : Schedules next ADC conversion and informs low-level
*                    driver about that time. Low-level driver must call
*                    this scheduler in that time.
*                    This function runs in interrupt!
*END*********************************************************************/

void _sensor_adc_scheduler(_mqx_uint time) { /* gets actual time as parameter */
    _mqx_int i;
    _mqx_uint wait;
    _mqx_int rem;
    
    if (Running_mask) { /* if there is any channel in the mask for measurement */
        _sensor_adc_hw_measure(Running_mask); /* measure channels */
    
        for (i = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) { /* update time of last measurement in channels */
            if (!pSensor_adc_ch[i] || !(Running_mask & (1 << pSensor_adc_ch[i]->g.source)))
                continue;
            
            pSensor_adc_ch[i]->g.time = time;
        }
    }
    
    /* look for closest AD conversion */
    for (i = 0, wait = -1, Running_mask = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
        if (pSensor_adc_ch[i] && pSensor_adc_ch[i]->g.runtime_flags & SENSOR_ADC_CHANNEL_RUNNING) {
            /* compute remaining time to next measurement:
            ** elapsed time from last measurement: time - pSensor_adc_ch[i]->time
            ** remaining time to next measurement: pSensor_adc_ch[i]->period - elapsed
            */
            rem = pSensor_adc_ch[i]->g.period - (time - pSensor_adc_ch[i]->g.time);
        
            if (rem < 0)
                rem = 0;
        
            if (rem < wait) {
                wait = rem;
                Running_mask = (1 << pSensor_adc_ch[i]->g.source);
            }
            else if (rem == wait) {
                Running_mask |= (1 << pSensor_adc_ch[i]->g.source);
            }
        }
    }
    
    /* test if any conversion should be performed next time */
    if (Running_mask) {
        if (!pSensor_adc->g.run) {
            _sensor_adc_hw_run();
            pSensor_adc->g.run = 1;
        }
    }
    else {
        _sensor_adc_hw_stop();
        pSensor_adc->g.run = 0;
    }
    
    /* inform low level driver that next dispatch call should be done in 'wait' time */
    _sensor_adc_hw_set_wait(wait);
}

/*FUNCTION*****************************************************************
* 
* Function Name    : void _sensor_intterrupt_adc_scheduler(_mqx_uint time)
* Returned Value   : 
* Comments         : Schedules next ADC conversion and informs low-level
*                    driver about that time. Low-level driver must call
*                    this scheduler in that time.
*                    This function runs in interrupt!
*END*********************************************************************/

void _sensor_intterrupt_adc_scheduler(_mqx_uint time)
{ /* gets actual time as parameter */
    _mqx_int i;
    _mqx_uint wait;
    _mqx_int rem;
    
    if (Running_mask) { /* if there is any channel in the mask for measurement */
        _sensor_adc_hw_measure(Running_mask); /* measure channels */
    
        for (i = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) { /* update time of last measurement in channels */
            if (!pSensor_adc_ch[i] || !(Running_mask & (1 << pSensor_adc_ch[i]->g.source)))
                continue;
            
            pSensor_adc_ch[i]->g.time = time;
        }
    }
    
    /* look for closest AD conversion */
    for (i = 0, wait = -1, Running_mask = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
        if (pSensor_adc_ch[i] && pSensor_adc_ch[i]->g.runtime_flags & SENSOR_ADC_CHANNEL_RUNNING) {
            /* compute remaining time to next measurement:
            ** elapsed time from last measurement: time - pSensor_adc_ch[i]->time
            ** remaining time to next measurement: pSensor_adc_ch[i]->period - elapsed
            */
            rem = pSensor_adc_ch[i]->g.period - (time - pSensor_adc_ch[i]->g.time);
        
            if (rem < 0)
                rem = 0;
        
            if (rem < wait) {
                wait = rem;
                Running_mask = (1 << pSensor_adc_ch[i]->g.source);
            }
            else if (rem == wait) {
                Running_mask |= (1 << pSensor_adc_ch[i]->g.source);
            }
        }
    }
    
    /* test if any conversion should be performed next time */
    if (Running_mask) {
        if (!pSensor_adc->g.run) {
            _sensor_adc_hw_run();
            pSensor_adc->g.run = 1;
        }
    }
    else {
        _sensor_adc_hw_stop();
        pSensor_adc->g.run = 0;
    }
    
    /* inform low level driver that next dispatch call should be done in 'wait' time */
    _sensor_adc_hw_set_wait(wait);
}


/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_adc_conv_end
* Returned Value   : 
* Comments         : This is called by low-level driver to inform generic
*                    driver, that some conversion was performed. Function
*                    moves stored data to the channel buffers.
*END*********************************************************************/

void _sensor_adc_conv_end(uint_32 *pResults_buf, SENSOR_ADC_HW_CHANNEL_MASK mask) {
    _mqx_int i;

    for (i = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
        if (!pSensor_adc_ch[i] 
          || !(pSensor_adc_ch[i]->g.runtime_flags & SENSOR_ADC_CHANNEL_RESUMED) 
          || !(mask & (1 << pSensor_adc_ch[i]->g.source))){
            continue;
        }
            
        if (pSensor_adc_ch[i]->g.init_flags & SENSOR_ADC_CHANNEL_ACCUMULATE){
            /* if the channel has ADC_CHANNEL_ACCUMULATE, then accumulate result */
            pSensor_adc_ch[i]->g.buffer_driver->result += pResults_buf[pSensor_adc_ch[i]->g.source];
        } else{
            /* else copy the result to the channel buffer */
            pSensor_adc_ch[i]->g.buffer_driver->result = pResults_buf[pSensor_adc_ch[i]->g.source];
        }

        if (!(pSensor_adc_ch[i]->g.init_flags & SENSOR_ADC_CHANNEL_ACCUMULATE) 
           || !(pSensor_adc_ch[i]->g.samples)) {
            /* move driver pointer to next index, circullarily */
            if (++pSensor_adc_ch[i]->g.buffer_driver >= pSensor_adc_ch[i]->g.buffer_end){
                pSensor_adc_ch[i]->g.buffer_driver = pSensor_adc_ch[i]->g.buffer_start;
            }

            /* if new value should overwrite old value in ring buffer, move read pointer to next value */
            if (pSensor_adc_ch[i]->g.buffer_driver == pSensor_adc_ch[i]->g.buffer_task) {
                if (++pSensor_adc_ch[i]->g.buffer_task >= pSensor_adc_ch[i]->g.buffer_end){
                    pSensor_adc_ch[i]->g.buffer_task = pSensor_adc_ch[i]->g.buffer_start;
                }
            }
        }
        
        if (pSensor_adc_ch[i]->g.samples) {
            pSensor_adc_ch[i]->g.samples--;
        }
        else {
#if MQX_USE_LWEVENTS
            /* if it is the last sample in the batch, then set event */
            if (pSensor_adc_ch[i]->g.complete_event)
                _lwevent_set(pSensor_adc_ch[i]->g.complete_event, 1); /* set completion event */
#endif
                
            if (pSensor_adc_ch[i]->g.init_flags & SENSOR_ADC_CHANNEL_MEASURE_ONCE)
                pSensor_adc_ch[i]->g.runtime_flags &= ~(SENSOR_ADC_CHANNEL_RUNNING);
            
            pSensor_adc_ch[i]->g.samples = pSensor_adc_ch[i]->g.samples_preset;
        }
    }
}

/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_adc_trigger
* Returned Value   : IO_OK
* Comments         : Starts channel measurement or more channels defined
*                    by trigger mask
*
*END*********************************************************************/

_mqx_int _sensor_adc_trigger(SENSOR_ADC_CHANNEL_GENERIC_PTR pChannel, SENSOR_ADC_TRIGGER_MASK mask) {
    _mqx_int i;
    _mqx_uint time;

//    _int_disable();
    time = _sensor_adc_hw_get_time();
    
    if (pChannel) {
        pChannel->time = time + pChannel->offset;
        pChannel->samples = pChannel->samples_preset;
        pChannel->runtime_flags |= SENSOR_ADC_CHANNEL_RUNNING | SENSOR_ADC_CHANNEL_RESUMED;
    }
    else {
        for (i = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
            if (pSensor_adc_ch[i] && (pSensor_adc_ch[i]->g.trigger & mask)) {
                pSensor_adc_ch[i]->g.time = time + pSensor_adc_ch[i]->g.offset;
                /* period was subtracted because next scheduling will add it */
                pSensor_adc_ch[i]->g.samples = pSensor_adc_ch[i]->g.samples_preset;
                pSensor_adc_ch[i]->g.runtime_flags |= SENSOR_ADC_CHANNEL_RUNNING | SENSOR_ADC_CHANNEL_RESUMED;
            }
        }
    }
    
    Running_mask = 0;
    _sensor_adc_scheduler(time);

    //_sensor_intterrupt_adc_scheduler(time);

//    _int_enable();
    return IO_OK;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_adc_pause
* Returned Value   : IO_OK
* Comments         : Pauses measurement of single channel, or more channels
*                    defined by trigger mask
*
*END*********************************************************************/

_mqx_int _sensor_adc_pause(SENSOR_ADC_CHANNEL_GENERIC_PTR pChannel, SENSOR_ADC_TRIGGER_MASK mask) {
    _mqx_int i;
    _mqx_uint time;

//    _int_disable();

    if (pChannel) {
        pChannel->runtime_flags &= ~(SENSOR_ADC_CHANNEL_RESUMED);
    }
    else {
        for (i = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
            if (pSensor_adc_ch[i] && (pSensor_adc_ch[i]->g.trigger & mask)) {
                pSensor_adc_ch[i]->g.runtime_flags &= ~(SENSOR_ADC_CHANNEL_RESUMED);
            }
        }
    }

//    _int_enable();
    return IO_OK;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_adc_resume
* Returned Value   : IO_OK
* Comments         : Resumes measurement of single paused channel, or more
*                    channels defined by trigger mask
*
*END*********************************************************************/

_mqx_int _sensor_adc_resume(SENSOR_ADC_CHANNEL_GENERIC_PTR pChannel, SENSOR_ADC_TRIGGER_MASK mask) {
    _mqx_int i;

//    _int_disable();

    if (pChannel) {
        pChannel->runtime_flags |= SENSOR_ADC_CHANNEL_RESUMED;
    }
    else {
        for (i = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
            if (pSensor_adc_ch[i] && (pSensor_adc_ch[i]->g.trigger & mask)) {
                pSensor_adc_ch[i]->g.runtime_flags |= SENSOR_ADC_CHANNEL_RESUMED;
            }
        }
    }

//    _int_enable();
    return IO_OK;
}

/*FUNCTION*****************************************************************
* 
* Function Name    : _sensor_adc_stop
* Returned Value   : IO_OK
* Comments         : Stops (resets) measurement of single channel, or more
*                    channels defined by trigger mask
*
*END*********************************************************************/

_mqx_int _sensor_adc_stop(SENSOR_ADC_CHANNEL_GENERIC_PTR pChannel, SENSOR_ADC_TRIGGER_MASK mask) {
    _mqx_int i;
    _mqx_uint time;

//    _int_disable();
    time = _sensor_adc_hw_get_time();
        
    if (pChannel) {
        pChannel->runtime_flags &= ~(SENSOR_ADC_CHANNEL_RUNNING);
    }
    else {
        for (i = 0; i < SENSOR_ADC_MAX_CHANNELS; i++) {
            if (pSensor_adc_ch[i] && (pSensor_adc_ch[i]->g.trigger & mask)) {
                    pSensor_adc_ch[i]->g.runtime_flags &= ~(SENSOR_ADC_CHANNEL_RUNNING);
            }
        }
    }
    
    Running_mask = 0;
    _sensor_adc_scheduler(time);

//    _int_enable();
    return IO_OK;
}

